import numpy as np

# This simple example solves the following constrainted optimization problem:
#
# argmin_{x} \sum_{i = 0}^{4} x_{i}
# s. t. 61 / x_{0}^3
#     + 37 / x_{1}^3
#     + 19 / x_{2}^3
#     + 7 / x_{3}^3
#     + 1 / x_{4}^3
#     - 1 < 0

# The API of the optimizer is entirely defined in the optlab module
import optlab

# Maximum number of iterations
maxIter = 10

# number of variables and constraints
numVariables = 5

# initial value of the variables in the design space
initialPoint = 5.0*np.ones(numVariables)
initialPoint = np.minimum(initialPoint,9.0)
initialPoint = np.maximum(initialPoint,2.0)

# lower bound for design variables (all set to 1)
lowerBound = np.ones(numVariables)

# upper bound for design variables (all set to 10)
upperBound = 10.0 * np.ones(numVariables)

# Initialize the MMA optimizer
optlab.mma.initialize(initialPoint, lowerBound, upperBound)

# Set some options for MMA
optlab.mma.option.setNumber('General.Verbosity', 4)
optlab.mma.option.setNumber('General.SaveHistory', 0)

# Get iteration count (here it will be 1 - could be different in case of restart)
it = optlab.mma.getOuterIteration()

while it <= maxIter:

    # get (copy of) current point
    x = np.array(optlab.mma.getCurrentPoint())
    print 'current point:', x

    # compute objective function and constraints
    # as well as their sensitivity with respect to design variables at `x'
    objective = np.sum(x)
    grad_objective = np.ones(numVariables)
    constraints = np.array([61.0/(x[0]**3.0)\
        + 37.0/(x[1]**3.0)\
        + 19.0/(x[2]**3.0)\
        + 7.0/(x[3]**3.0)\
        + 1.0/(x[4]**3.0)\
        - 1.0])
    grad_constraints = np.zeros(numVariables * 1)
    grad_constraints[0] = -3.0*61.0/(x[0]**4.0)
    grad_constraints[1] = -3.0*37.0/(x[1]**4.0)
    grad_constraints[2] = -3.0*19.0/(x[2]**4.0)
    grad_constraints[3] = -3.0*7.0/(x[3]**4.0)
    grad_constraints[4] = -3.0*1.0/(x[4]**4.0)

    print 'objective:', objective
    print 'constraints:', constraints
    print 'gradient of objective:', grad_objective
    print 'gradient of constraints:', grad_constraints

    # call MMA and update the current point
    optlab.mma.updateCurrentPoint(constraints,grad_objective,grad_constraints)

    print '*'*50
    print 'iteration: ', it, ' objective: ', objective
    print '*'*50

    it = optlab.mma.getOuterIteration()

# This should be called at the end
optlab.mma.finalize()
